@@ -196,9 +196,17 @@ proc redraw {} {
     global Machines
     global after_token
     global Data
+    global TotalData
+
     set after_token ""
     set index 0
     set persec_total 0
+    set busy_slaves_total 0
+    set avail_slaves_total 0
+    set msgs_per_sec_total 0
+    set ms_per_scan_total 0
+    set num_machines 0
+
     foreach m $Machines {
 	set mach [lindex $m 0]
 	if {![info exists Data($mach,busy)]} {
@@ -231,11 +239,41 @@ proc redraw {} {
 	set Data($mach,persec) [graph [expr 1.0/3] [expr 2.0/3] 0 auto $Data($mach,persec) $index "Msgs/Sec" 2 green 1]
 	set Data($mach,time) [graph [expr 2.0/3] 1 0 auto $Data($mach,time) $index "ms/scan" 3 blue 1]
 	incr index
+
+	if {$ms0 > 0 || $Data($mach,busy_snap) > 0 || $msg0 > 0} {
+	    incr num_machines
+	    incr busy_slaves_total $Data($mach,busy_snap)
+	    incr avail_slaves_total $Data($mach,total_slaves);
+	    incr ms_per_scan_total $ms0
+	}
     }
+    lappend TotalData(busy) $busy_slaves_total
+    if {$num_machines > 0} {
+	lappend TotalData(time) [expr 1.0 * $ms_per_scan_total / (1.0 * $num_machines)]
+    } else {
+	lappend TotalData(time) 0
+    }
+    lappend TotalData(persec) $persec_total
+
+    incr index
+    set TotalData(busy) [graph 0 [expr 1.0/3] 0 $avail_slaves_total $TotalData(busy) $index "Busy" 1 red 1]
+    set TotalData(persec) [graph [expr 1.0/3] [expr 2.0/3] 0 auto $TotalData(persec) $index "Msgs/Sec" 2 green 1]
+    set TotalData(time) [graph [expr 2.0/3] 1 0 auto $TotalData(time) $index "ms/scan" 3 blue 1]
+
+    set msgs_per_sec_total $persec_total
     set hour [human_number [expr $persec_total * 3600.0]]
     set day  [human_number [expr $persec_total * 86400.0]]
     set persec_total [strip_zeros [format "%.1f" $persec_total]]
     .top.c configure -text "Total throughput $persec_total/s = $hour/hour = $day/day"
+    set l [string length $avail_slaves_total]
+    set busy_slaves_total [format "%${l}d" $busy_slaves_total]
+    .top.busytotal configure -text "$busy_slaves_total/$avail_slaves_total"
+    .top.persectotal configure -text [strip_zeros [format "%.1f" $msgs_per_sec_total]]
+    if {$num_machines > 0} {
+	.top.avgtime configure -text [strip_zeros [format "%.0f" [expr 1.0 * $ms_per_scan_total / (1.0 * $num_machines)]]]
+    } else {
+	.top.avgtime configure -text "--"
+    }
     update
 }
 
@@ -281,6 +319,10 @@ proc graph { start_frac end_frac min max data index label tag fill_color line_wi
     if {$num_gridlines > 10} {
 	set num_gridlines 10
     }
+    if {$num_gridlines < 1} {
+	set num_gridlines 1
+    }
+
     set delta [nicenum [expr $diff / $num_gridlines] 1]
     foreach point $data {
 	set y [expr $point - $min]
@@ -362,6 +404,7 @@ proc reconfigure {} {
     catch { destroy .top.time }
     catch { destroy .top.c }
     catch { destroy .top.name }
+
     label .top.name -text "Machine Name"
     label .top.busy -text "Busy Slaves" -foreground "#A00000"
     label .top.persec -text "Msgs/s" -foreground "#00A000"
@@ -381,16 +424,41 @@ proc reconfigure {} {
     catch { destroy .top.time$index}
     catch { destroy .top.c$index}
 
+    incr index
+    # Bottom row of labels
+    catch { destroy .top.busytotal }
+    catch { destroy .top.persectotal }
+    catch { destroy .top.avgtime }
+    catch { destroy .top.totalrow }
+    catch { destrop .top.c$index }
+
+    # Mop up total window if a machine has been deleted
+    set i [expr $index + 1]
+    catch { destroy .top.c$i }
+
+    label .top.totalrow -text "Totals:"
+    label .top.busytotal
+    label .top.persectotal
+    label .top.avgtime
+    canvas .top.c$index -width 400 -height 60 -takefocus 0 -borderwidth 0 -background #FFFFF0 -highlightthickness 0
+
+    grid .top.totalrow -row $index -column 0 -sticky new
+    grid .top.busytotal -row $index -column 1 -sticky new
+    grid .top.persectotal -row $index -column 2 -sticky new
+    grid .top.avgtime -row $index -column 3 -sticky new
+    grid .top.c$index -row $index -column 4 -sticky nsew -pady 1
+    grid rowconfigure .top $index -weight 3
+    incr index
     # Now a spot for adding a new machine...
     catch { destroy .top.newlab }
     catch { destroy .top.new }
 
     label .top.newlab -text "Add Machine: "
     entry .top.new -width 20
-    grid .top.newlab -row [expr $index + 1] -column 0
-    grid .top.new -row [expr $index + 1] -column 1 -columnspan 4 -sticky ew
+    grid .top.newlab -row $index -column 0
+    grid .top.new -row $index -column 1 -columnspan 4 -sticky ew
     bind .top.new <Return> interactive_add_machine
-    grid rowconfigure .top [expr $index + 1] -weight 0
+    grid rowconfigure .top $index -weight 0
 
     grid columnconfigure .top 0 -weight 0
     grid columnconfigure .top 1 -weight 0
@@ -548,7 +616,7 @@ proc grid_machine { m index } {
     label .top.busy$index -text ""
     label .top.persec$index -text ""
     label .top.time$index -text ""
-    canvas .top.c$index -width 400 -height 80 -takefocus 0 -borderwidth 0 -background white -takefocus 0 -highlightthickness 0
+    canvas .top.c$index -width 400 -height 60 -takefocus 0 -borderwidth 0 -background white -highlightthickness 0
     .top.c$index create text 2 2 -anchor nw -text "" -tags statusText
     grid .top.name$index -row $row -column 0 -sticky new
     grid .top.busy$index -row $row -column 1 -sticky new
